#!/usr/bin/env python
# Ubuntu CI Engine
# Copyright 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
from __future__ import unicode_literals


import json
import logging
import sys
import uuid


from britney import queues
from ci_utils import (
    amqp_utils,
    dump_stack,
    unit_config,
)
# FIXME: We need a better overall story for the available images, for now, just
# use the basic ones -- vila 2014-09-23
from ci_utils.testing import features
# FIXME: Yet, a simple and direct way to bootstrap would be to just create
# uci-engine-britney-{series}-{arch} images. -- vila 2014-09-23

# I.e.: https://app.asana.com/0/8312550429058/16178663570516
# 'minimal images for britney'


class BritneyRequestHandler(object):
    """Process a britney test request.

    This is fed by messages received on a queue or directly for tests by
    calling the ``handle()`` method.
    """

    def __init__(self, test_runner_queue, output_queue):
        """Britney requests handler.

        :param test_runner_queue: The queue where the test runner requests
            should be sent.

        :param output_queue_name: The rabbit queue name where the test runner
            will send progress and results.

        """
        super(BritneyRequestHandler, self).__init__()
        self.test_runner_queue = test_runner_queue
        self.output_queue = output_queue
        self.auth_conf = unit_config.get_auth_config()

    def get_image_id(self, series):
        if unit_config.is_hpcloud(self.auth_conf.get('glance_os_auth_url')):
            test_images = features.hpcloud_test_images
        else:
            test_images = features.canonistack_test_images
        return test_images[series]

    def handle(self, params):
        series = params['series']
        architecture = params['architecture']
        package = params['package']
        # The output queue can be overridden for a given request (for tests and
        # debug purposes).
        output_queue = params.get('output_queue', self.output_queue)
        report_queue = params.get('report_queue', None)
        # Only amd64 architecture is supported for now, everything else is
        # dropped on the floor as we can't send feedback to the user at this
        # point (the data flow is: britney.requests -> test_runner ->
        # britney.results). See
        # https://app.asana.com/0/8312550429058/12699749556376 'reporting test
        # requests errors'
        if architecture == 'amd64':
            params = dict(ticket_id=str(uuid.uuid1()),
                          progress_trigger=output_queue,
                          series=series,
                          image_id=self.get_image_id(series),
                          ppa_list=[],
                          package_list=[package])
            if report_queue is not None:
                params['report_queue'] = report_queue
            self.test_runner_queue.publish(params)


class BritneyRequestsQueue(object):
    """The queue receiving the britney test requests.

    The requests are forwarded to the request handler.
    """

    def __init__(self, queue_name, request_handler):
        """Britney test requests queue.

        :param queue_name: The queue name where the test requests are received.

        :param request_handler: The handler that processes the requests.
        """
        self.queue_name = queue_name
        self.request_handler = request_handler

    def consume(self):
        conf = amqp_utils.get_config()
        # Process all messages and delete the queue when done
        amqp_utils.process_queue(conf, self.queue_name, self.handle_request,
                                 delete=True)

    def handle_request(self, msg):
        body = json.loads(msg.body)
        ret = self.request_handler.handle(body)
        msg.channel.basic_ack(msg.delivery_tag)
        return ret


if __name__ == '__main__':
    dump_stack.install_stack_dump_signal()
    logging.root.setLevel(logging.INFO)
    logging.info('Waiting for messages. ^C to exit.')
    tr_queue = queues.PublisherQueue(amqp_utils.TEST_RUNNER_QUEUE)
    request_handler = BritneyRequestHandler(tr_queue, 'britney.results')
    # Build the britney requests queue subscribing to the rabbit queue and
    # proving the handler that will forward to the test runner queue.
    queue = BritneyRequestsQueue('britney.requests', request_handler)
    queue.consume()
    sys.exit(0)
