#!/usr/bin/env python
# Ubuntu Continuous Integration Engine
# Copyright 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import json
import logging
import time
import signal
import sys

import os
from novaclient import exceptions
from novaclient.v1_1 import client
from ci_utils import (
    amqp_utils,
    unit_config,
    health_check,
    ticket_states,
)


class Timeout(Exception):
    pass


TIMEOUT = 60 * 7  # 420 seconds


def timeout_handler(sig_num, frame):
    raise Timeout('No worker responding after {} seconds'.format(TIMEOUT))


signal.signal(signal.SIGALRM, timeout_handler)
signal.alarm(TIMEOUT)


class MessageSync(object):
    progress = logging.getLogger('PROGRESS_TRIGGER')

    def __init__(self):
        self.data = None

    def on_message(self, msg):
        data = json.loads(msg.body)
        state = data['state']

        # print "state" as a label for the rest of the items in the list
        params = {x: y for (x, y) in data.iteritems() if x != 'state'}
        if params:
            self.progress.info('%s: %s', state, params)
        else:
            self.progress.info(state)

        msg.channel.basic_ack(msg.delivery_tag)
        if data.get('exit'):
            self.data = data
            # this will kick us out of the run-forever worker loop
            msg.channel.basic_cancel(msg.consumer_tag)

    @staticmethod
    def drain(queue):
        sync = MessageSync()
        cfg = amqp_utils.get_config()
        amqp_utils.process_queue(cfg, queue, sync.on_message, delete=True)
        return sync.data


def build_nova_client():
    args = [unit_config.get('glance_auth_user'),
            unit_config.get('glance_auth_password'),
            unit_config.get('glance_auth_tenant_name'),
            unit_config.get('glance_auth_url')]
    kwargs = {'region_name': unit_config.get('glance_auth_region'),
              'service_type': 'compute'}
    return client.Client(*args, **kwargs)


def list_nova_image():
    nova = build_nova_client()
    try:
        return nova.images.list()
    except exceptions.NotFound:
        raise Exception('Images cannot be listed')


def get_hp_image_id():
    image_list = list_nova_image()
    for image_name in image_list:
        if ('Ubuntu Server 14.04' in image_name.name
           and 'deprecated' not in image_name.name):
            return image_name.name


def get_canonistack_image_id():
    image_list = list_nova_image()
    for image_name in image_list:
        if ('ubuntu-released'.lower() in image_name.name.lower()
           and 'server' in image_name.name.lower()
           and 'trusty' in image_name.name.lower()):
            return image_name.name


def check_worker():
    queue = 'testrun-test-%d' % time.time()
    step = str(ticket_states.TicketWorkflowStep.IMAGE_TESTING)
    image_id = ""
    auth_url = unit_config.get('glance_auth_url')
    if 'hpcloud' in auth_url:
        image_id = get_hp_image_id()
    elif 'canonistack' in auth_url:
        image_id = get_canonistack_image_id()
    else:
        logging.error("Test does not support this cloud yet")
        sys.exit(1)
    params = {
        'ticket_id': 'test-runner-test',
        'image_id': image_id,
        'series': 'trusty',
        'package_list': ['libpng'],
        'ppa_list': [],
        'progress_trigger': queue,
    }
    amqp_utils.send(
        amqp_utils.TEST_RUNNER_QUEUE, json.dumps(params), True)
    try:
        data = MessageSync.drain(queue)
        output = 'data returned: {}'.format(data)
        result = 'COMPLETED' == data['state'] and 'PASSED' == data['result']
    except Timeout as e:
        result = False
        output = unicode(e)
    return result, output


def main():
    with health_check.HealthCheck('test_runner') as check:
        result, output = check_worker()
        if result:
            check.write_result_ok()
        else:
            check.write_result_failed(output)


if __name__ == '__main__':
    main()
