# Ubuntu CI Engine
# Copyright 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from cStringIO import StringIO
import logging
import os
import unittest

from ucitests import (
    assertions,
    fixtures as uci_fixtures,
)
from ucivms.tests import fixtures as vms_fixtures
from ci_utils import (
    amqp_utils,
    unit_config,
)
from ci_utils.testing import (
    features,
    fixtures,
)
from tstrun import (
    run_worker,
    testbed,
)


@features.requires(features.nova_creds)
@features.requires(features.nova_compute)
class TestWorkerHandleRequest(unittest.TestCase):

    def setUp(self):
        super(TestWorkerHandleRequest, self).setUp()
        # A logger for the test runner worker
        self.logger_stream = logging.StreamHandler(StringIO())
        self.logger_stream.setFormatter(logging.Formatter('%(message)s'))
        self.logger = logging.getLogger()
        self.logger.addHandler(self.logger_stream)
        # The fake rabbit input queue
        self.queue = fixtures.FakeTaskQueue('test')
        # The data store and uci-vms need disk access
        vms_fixtures.isolate_from_disk(self)
        self.ds_path = os.path.join(self.uniq_dir, 'data_store')
        # We don't need a config for the data store
        self.data_store = fixtures.FakeDataStore(self.ds_path)

        def create_data_store(component):
            return self.data_store

        self.ds_factory = create_data_store
        self.conf = unit_config.get_auth_config()

    def get_ppa_list(self):
        # We're testing only the test runner so there is no ppa assigned to the
        # ticket, only the master ppa is relevant.
        return [self.conf.get('master_ppa')]

    def get_image_id(self, series):
        if unit_config.is_hpcloud(self.conf.get('glance_os_auth_url')):
            test_images = features.hpcloud_test_images
        else:
            test_images = features.canonistack_test_images
        return test_images[series]

    def assertTestRun(self, expected_artifacts, params, worker=None):
        if worker is None:
            worker = run_worker.TestRunnerWorker(self.ds_factory)
        worker = run_worker.TestRunnerWorker(self.ds_factory)
        self.queue.publish(params)
        returns = self.queue.subscribe(worker.handle_request, self.logger)
        assertions.assertLength(self, 1, returns)
        self.assertTrue(self.queue.is_empty())
        self.assertEquals(expected_artifacts, self.data_store.list_files())
        return returns

    def test_two_packages_two_testbeds(self):
        # We use precise where no tests are defined for libpng nor adduser to
        # speed things up a bit without reducing coverage. (~244s instead of
        # 560~ on a single sample (YMMV, high variations there)).
        series = 'precise'
        params = dict(ticket_id=series, progress_trigger='progress',
                      series=series,
                      image_id=self.get_image_id(series),
                      ppa_list=self.get_ppa_list(),
                      package_list=['libpng', 'adduser'])
        returns = self.assertTestRun(
            ['adduser-summary.log', 'adduser-syslog.log',
             'adduser-testbed-cloud-init.log',
             'adduser.log',
             'adduser.subunit',
             'libpng-summary.log', 'libpng-syslog.log',
             'libpng-testbed-cloud-init.log',
             'libpng.log',
             'libpng.subunit'],
            params)
        (retcode, results) = returns[0]
        self.assertEqual(amqp_utils.progress_failed, retcode)
        self.assertEquals(
            '*                    SKIP no tests in this package\n',
            self.data_store.get_file('libpng-summary.log'))
        self.assertEquals(
            '*                    SKIP no tests in this package\n',
            self.data_store.get_file('adduser-summary.log'))

    def test_handle_request_precise_image(self):
        series = 'precise'
        params = dict(ticket_id=series, progress_trigger='progress',
                      series=series,
                      image_id=self.get_image_id(series),
                      ppa_list=self.get_ppa_list(),
                      package_list=['libpng'])
        returns = self.assertTestRun(
            ['libpng-summary.log', 'libpng-syslog.log',
             'libpng-testbed-cloud-init.log', 'libpng.log',
             'libpng.subunit'],
            params)
        (retcode, results) = returns[0]
        self.assertEqual(amqp_utils.progress_failed, retcode)
        # We need to check the content here as the worker will put an error
        # message there if it cannot get them from the testbed (so the user get
        # *some* feedback).
        self.assertEquals(
            '*                    SKIP no tests in this package\n',
            self.data_store.get_file('libpng-summary.log'))

    def test_handle_request_saucy_image(self):
        series = 'saucy'
        params = dict(ticket_id=series, progress_trigger='progress',
                      series=series,
                      image_id=self.get_image_id(series),
                      ppa_list=self.get_ppa_list(),
                      package_list=['libpng'])
        returns = self.assertTestRun(
            ['libpng-summary.log', 'libpng-syslog.log',
             'libpng-testbed-cloud-init.log', 'libpng.log',
             'libpng.subunit'],
            params)
        (retcode, results) = returns[0]
        self.assertEqual(amqp_utils.progress_completed, retcode)
        # We need to check the content here as the worker will put an error
        # message there if it cannot get them from the testbed (so the user get
        # *some* feedback).
        self.assertEquals('build                PASS\n',
                          self.data_store.get_file('libpng-summary.log'))

    def test_handle_request_trusty_image(self):
        series = 'trusty'
        params = dict(ticket_id=series, progress_trigger='progress',
                      series=series,
                      image_id=self.get_image_id(series),
                      ppa_list=self.get_ppa_list(),
                      package_list=['libpng'])
        returns = self.assertTestRun(
            ['libpng-summary.log', 'libpng-syslog.log',
             'libpng-testbed-cloud-init.log', 'libpng.log',
             'libpng.subunit'],
            params)
        (retcode, results) = returns[0]
        self.assertEqual(amqp_utils.progress_completed, retcode)
        # We need to check the content here as the worker will put an error
        # message there if it cannot get them from the testbed (so the user get
        # *some* feedback).
        self.assertEquals('build                PASS\n',
                          self.data_store.get_file('libpng-summary.log'))

    def test_teardown_dont_break_test_run(self):
        series = 'trusty'
        params = dict(ticket_id=series, progress_trigger='progress',
                      series=series,
                      image_id=self.get_image_id(series),
                      ppa_list=self.get_ppa_list(),
                      package_list=['libpng'])
        worker = run_worker.TestRunnerWorker(self.ds_factory)

        def broken_teardown(test_bed):
            self.addCleanup(test_bed.nova.delete_server, test_bed.instance.id)
            if test_bed.floating_ip is not None:
                self.addCleanup(test_bed.nova.delete_floating_ip,
                                test_bed.floating_ip)
            raise AssertionError('Boom !')

        uci_fixtures.patch(self, testbed.TestBed, 'teardown', broken_teardown)
        returns = self.assertTestRun(
            ['libpng-summary.log', 'libpng-syslog.log',
             'libpng-testbed-cloud-init.log', 'libpng.log',
             'libpng.subunit'],
            params, worker)
        (retcode, results) = returns[0]
        self.assertEqual(amqp_utils.progress_completed, retcode)
