#!/usr/bin/env python
# Ubuntu Continuous Integration Engine
# Copyright 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import json
import os
import unittest


from ci_utils import amqp_utils
from ci_utils.testing import features
import deploy
import deployers
from ucitests import (
    assertions,
)


from britney import process_results


class TestProcessRequests(deployers.DeployerTest):

    def setUp(self):
        super(TestProcessRequests, self).setUp()
        status = deploy.juju_status()
        # We need the right deployment
        try:
            br = status['services']['ci-airline-for-britney-requests']['units']
            br['ci-airline-for-britney-requests/0']
        except KeyError:
            self.skipTest('Not a valid ci-airline-for-britney deployment')
        units = status['services']['ci-airline-rabbit']['units']
        rabbit_ip = units['ci-airline-rabbit/0']['public-address']
        deployers.expose_rabbit(self, rabbit_ip, 'tester', 's3cr3t')
        # We delay the import so it won't fail when the 'unit_config' file
        # is not available
        from ci_utils import unit_config
        conf = unit_config.get_auth_config()
        if unit_config.is_hpcloud(conf.get('glance_os_auth_url')):
            self.test_images = features.hpcloud_test_images
        else:
            self.test_images = features.canonistack_test_images

    def drain_tr_queue(self, queue_name):
        messages = []

        def on_message(msg):
            body = json.loads(msg.body)
            messages.append(body)
            msg.channel.basic_ack(msg.delivery_tag)
            if body.get('exit'):
                # this will kick us out of the run-forever worker loop, relying
                # on the worker last message setting 'exit'
                msg.channel.basic_cancel(msg.consumer_tag)

        conf = amqp_utils.get_config()
        # Process all messages and delete the queue when done
        amqp_utils.process_queue(conf, queue_name, on_message, delete=True)
        return messages

    def assertStatusMsg(self, expected, msg):
        self.assertEqual('STATUS', msg['state'])
        without_state = msg.copy()
        del without_state['state']
        self.assertDictEqual(expected, without_state)

    def get_image_id(self, series):
        return self.test_images[series]

    def test_tr_process_test_request(self):
        # Exercise the britney API: feed a test request on the britney requests
        # queue and ensures the test runner receive a translated request and
        # process it properly. We force a specific result queue to avoid
        # involving the britney results service.
        result_queue_name = 'britney.test_tr_process_test_request'
        series = 'precise'
        architecture = 'amd64'
        params = dict(series=series,
                      architecture=architecture,
                      package='libpng',
                      output_queue=result_queue_name)
        amqp_utils.send('britney.requests', json.dumps(params),
                        raise_errors=True)
        msgs = self.drain_tr_queue(result_queue_name)
        assertions.assertLength(self, 6, msgs)
        # The test runner received the translated request
        tr_params = msgs[0]
        self.assertEquals(self.get_image_id(series), tr_params['image_id'])
        self.assertEquals([], tr_params['ppa_list'])
        self.assertEquals(['libpng'], tr_params['package_list'])
        self.assertEquals(series, tr_params['series'])
        ticket_id = tr_params['ticket_id']
        setup = 'Setting up the testbed for ticket {}'.format(ticket_id)
        self.assertStatusMsg(dict(message=setup), msgs[1])
        self.assertStatusMsg(dict(message='The test bed is ready'), msgs[2])
        self.assertStatusMsg(dict(message='Testing libpng'), msgs[3])
        completed = 'Test completed for ticket {}'.format(ticket_id)
        self.assertStatusMsg(dict(message=completed), msgs[4])
        final = msgs[5]
        # No tests for libpng on precise
        self.assertEqual('FAILED', final['result'])
        self.assertEqual('COMPLETED', final['state'])

    def drain_report_queue(self, queue_name):

        messages = []

        def on_message(msg):
            # We expect a single message here so we just ack it and leave the
            # loop
            body = json.loads(msg.body)
            messages.append(body)
            msg.channel.basic_ack(msg.delivery_tag)
            msg.channel.basic_cancel(msg.consumer_tag)

        conf = amqp_utils.get_config()
        # Process all messages and delete the queue when done
        amqp_utils.process_queue(conf, queue_name, on_message, delete=True)
        return messages

    def test_process_test_and_result_request(self):
        # Exercise the britney API: feed a test request on the britney requests
        # queue and check that the britney result is processed.
        report_queue_name = 'test-britney.results'
        series = 'precise'
        architecture = 'amd64'
        params = dict(series=series,
                      architecture=architecture,
                      package='libpng',
                      report_queue=report_queue_name)
        amqp_utils.send('britney.requests', json.dumps(params),
                        raise_errors=True)
        msgs = self.drain_report_queue(report_queue_name)
        # The results are in the britney swift container
        assertions.assertLength(self, 1, msgs)
        # Check the swift container content
        dir_name = msgs[0]['result_at']
        log_file_name = os.path.join(dir_name, 'libpng.log')
        # We delay the import so it won't fail when the 'unit_config' file
        # is not available
        from ci_utils import unit_config
        britney_data_store = process_results.BritneyDataStore(
            'britney.results', unit_config.get_auth_config(), public=True)
        self.assertIn(log_file_name, britney_data_store.list_files())


if __name__ == "__main__":
    unittest.main()
