#!/usr/bin/env python
# Ubuntu Continuous Integration Engine
# Copyright 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import json
import uuid
import unittest
import deployers
import deploy

from ci_utils import amqp_utils
from ucitests import (
    assertions,
    fixtures,
)

RABBIT_SERVICE = 'ci-airline-rabbit'


def run_rabbit_cmd(args):
    deployers.juju_run(RABBIT_SERVICE,
                       'sudo rabbitmqctl {}'.format(' '.join(args)))


def create_rabbit_user(user_name, password=None):
    if password is None:
        password = user_name
    run_rabbit_cmd(['add_user', user_name, password])
    run_rabbit_cmd(['set_permissions', '-p', '/', user_name, '".*" ".*" ".*"'])


def delete_rabbit_user(user_name):
    run_rabbit_cmd(['delete_user', user_name])


class TestPPACreator(deployers.DeployerTest):
    """Integration tests for ppacreator service run on a juju deployment"""

    def setUp(self):
        super(TestPPACreator, self).setUp()
        status = deploy.juju_status()
        units = status['services']['ci-airline-rabbit']['units']
        self.unit = 'ci-airline-rabbit/0'
        self.public_ip = units[self.unit]['public-address']
        # Expose rabbit service for this test.
        deployers.juju_expose(RABBIT_SERVICE)
        self.addCleanup(deployers.juju_expose, RABBIT_SERVICE, False)

        class AmqpConfig(object):

            def __init__(inner):
                inner.AMQP_HOST = self.public_ip
                inner.AMQP_VHOST = '/'
                inner.AMQP_USER = 'tester'
                inner.AMQP_PASSWORD = 's3cr3t'
        fixtures.patch(self, amqp_utils, 'get_config', AmqpConfig)
        create_rabbit_user('tester', 's3cr3t')
        self.addCleanup(delete_rabbit_user, 'tester')

    def drain_queue(self, queue_name):
        self.messages = []

        def on_message(msg):
            body = json.loads(msg.body)
            self.messages.append(body)
            msg.channel.basic_ack(msg.delivery_tag)
            if body.get('exit'):
                # this will kick us out of the run-forever worker loop, relying
                # on the worker last message setting 'exit'
                msg.channel.basic_cancel(msg.consumer_tag)

        conf = amqp_utils.get_config()
        # Process all messages and delete the queue when done
        amqp_utils.process_queue(conf, queue_name, on_message, delete=True)
        return self.messages

    def assertStatusMsg(self, expected, msg):
        self.assertEqual('STATUS', msg['state'])
        without_state = msg.copy()
        del without_state['state']
        self.assertDictEqual(expected, without_state)

    def test_worker_running(self):
        """Ensure the ppacreator worker is deployed and running."""
        self.assert_job_running('ci-airline-ppa-creator-worker')

    def test_create_ppa(self):
        """Ensure that ppa's can be created or their name can
           be retrieved if they are already existing"""
        unique_id = uuid.uuid1()
        progress_queue_name = 'progress-trigger'
        params = dict(
            name='ppacreator-inttest-{}'.format(unique_id),
            displayname='ppacreator-inttest-{}'.format(unique_id),
            description='int-test-ppa-description',
            progress_trigger=progress_queue_name,
            ticket_id='int-test-ppa-ticket-id',
            architectures=('all'),
            series='precise')

        amqp_utils.send(amqp_utils.PPA_CREATOR_QUEUE, json.dumps(params),
                        raise_errors=True)
        msgs = self.drain_queue(progress_queue_name)
        assertions.assertLength(self, 3, msgs)
        self.assertStatusMsg({u'message': u'Creating PPA {} for ticket, '
                              'int-test-ppa-ticket-id'.format(params['name'])},
                             msgs[1])

        final = msgs[2]
        self.assertEqual('COMPLETED', final['state'])
        self.assertEqual('PASSED', final['result'])

if __name__ == "__main__":
    unittest.main()
